// blob code from: Daniel Shiffman
// http://codingtra.in
// http://patreon.com/codingtrain
// Code for: https://youtu.be/r0lvsMPGEoY

//are there two blobs?
//what is blob size 1?
//or: what is blob size 1+2?
//if there are two blobs but big: shock, two but small: smile, one big: neutral, one small: kissy

PImage happy;
PImage kissy;
PImage neutral;
PImage shock;

import processing.video.*;

Capture video;

int blobCounter = 0;

boolean showImage = false;
boolean oneBlob = true;

//int maxLife = 200;

color trackColor; 
float threshold = 40;
float distThreshold = 10;

ArrayList<Blob> blobs = new ArrayList<Blob>();

void setup() {
  size(640, 640);
  String[] cameras = Capture.list();
  printArray(cameras);
  video = new Capture(this, cameras[3]);
  video.start();
  // 183.0 12.0 83.0
  trackColor = color(190, 48, 80);
  
  happy = loadImage("happy_emoji.png");
  shock = loadImage("shock_emoji.png");
  kissy = loadImage("kissy_emoji.png");
  neutral = loadImage("neutral_emoji.png");
}

void captureEvent(Capture video) {
  video.read();
}

void keyPressed() {
  if (key == 'a') {
    distThreshold+=5;
  } else if (key == 'z') {
    distThreshold-=5;
  }
  if (key == 's') {
    threshold+=5;
  } else if (key == 'x') {
    threshold-=5;
  }
  if (key == 'i'){
    if (showImage){
      showImage = false;
    }else {
      showImage = true;
    }
  }
}

void draw() {
  video.loadPixels();
  
  if (showImage){
  image(video, 0, 0);
  }
  if (showImage == false){
    background(0);
  }

  ArrayList<Blob> currentBlobs = new ArrayList<Blob>();

  // Begin loop to walk through every pixel
  for (int x = 0; x < video.width; x++ ) {
    for (int y = 0; y < video.height; y++ ) {
      int loc = x + y * video.width;
      // What is current color
      color currentColor = video.pixels[loc];
      float r1 = red(currentColor);
      float g1 = green(currentColor);
      float b1 = blue(currentColor);
      float r2 = red(trackColor);
      float g2 = green(trackColor);
      float b2 = blue(trackColor);

      float d = distSq(r1, g1, b1, r2, g2, b2); 

//is point near blob, if not make a new blob, if yes add to blob --while checking pixels
      if (d < threshold*threshold) {

        boolean found = false;
        for (Blob b : currentBlobs) {
          if (b.isNear(x, y)) {
            b.add(x, y);
            found = true;
            break;
          }
        }

        if (!found) {
          Blob b = new Blob(x, y);
          currentBlobs.add(b);
        }
      }
    }
  }

//take out small blobs
  for (int i = currentBlobs.size()-1; i >= 0; i--) {
    if (currentBlobs.get(i).size() < 500) {
      currentBlobs.remove(i);
    }
  }

  // There are no blobs!
  if (blobs.isEmpty() && currentBlobs.size() > 0) {
    println("Adding blobs!");
    for (Blob b : currentBlobs) {
      b.id = blobCounter;
      blobs.add(b);
      blobCounter++;
    }
  } else if (blobs.size() <= currentBlobs.size()) {
    // Match whatever blobs you can match
    for (Blob b : blobs) {
      float recordD = 1000;
      Blob matched = null;
      for (Blob cb : currentBlobs) {
        PVector centerB = b.getCenter();
        PVector centerCB = cb.getCenter();         
        float d = PVector.dist(centerB, centerCB);
        if (d < recordD && !cb.taken) {
          recordD = d; 
          matched = cb;
        }
      }
      matched.taken = true;
      b.become(matched);
    }

    // Whatever is leftover make new blobs
    for (Blob b : currentBlobs) {
      if (!b.taken) {
        b.id = blobCounter;
        blobs.add(b);
        blobCounter++;
      }
    }
  } else if (blobs.size() > currentBlobs.size()) {
    for (Blob b : blobs) {
      b.taken = false;
    }


    // Match whatever blobs you can match
    for (Blob cb : currentBlobs) {
      float recordD = 1000;
      Blob matched = null;
      for (Blob b : blobs) {
        PVector centerB = b.getCenter();
        PVector centerCB = cb.getCenter();         
        float d = PVector.dist(centerB, centerCB);
        if (d < recordD && !b.taken) {
          recordD = d; 
          matched = b;
        }
      }
      if (matched != null) {
        matched.taken = true;
        matched.become(cb);
      }
    }

    for (int i = blobs.size() - 1; i >= 0; i--) {
      Blob b = blobs.get(i);
      if (!b.taken) {
        blobs.remove(i);
      }
    }
  }
  
    

  for (Blob b : blobs) {
    //b.show();
    if ((currentBlobs.size()>1)&&(showImage == false)){
      for (int i = currentBlobs.size()-1; i >= 0; i--) {
        if (currentBlobs.get(i).size() > 1400) {
          image(shock,60,60);
        }
        else {
          image(happy,60,60);
        }
      }
    }
    else {
    if ((b.size()>1300)&&(showImage == false)){
      image(neutral,60,60);
    }
    else if ((b.size()<1300)&&(showImage == false)){
      image(kissy,60,60);
    }
    }
  }

  textAlign(RIGHT);
  fill(0);
  text(currentBlobs.size(), width-10, 40);
  text(blobs.size(), width-10, 80);
  textSize(24);
  text("color threshold: " + threshold, width-10, 50);  
  text("distance threshold: " + distThreshold, width-10, 25);
}


float distSq(float x1, float y1, float x2, float y2) {
  float d = (x2-x1)*(x2-x1) + (y2-y1)*(y2-y1);
  return d;
}


float distSq(float x1, float y1, float z1, float x2, float y2, float z2) {
  float d = (x2-x1)*(x2-x1) + (y2-y1)*(y2-y1) +(z2-z1)*(z2-z1);
  return d;
}

void mousePressed() {
  // Save color where the mouse is clicked in trackColor variable
  int loc = mouseX + mouseY*video.width;
  trackColor = video.pixels[loc];
  println(red(trackColor), green(trackColor), blue(trackColor));
}
